<?php // config.php

#   File was released under GNU General Public License 3
#   Articles Copyright (C) 2024  Wloff Apps, Services, Development
#   This program comes with ABSOLUTELY NO WARRANTY;
#=================================================================

# configuration file, settings
# variable definitions
# function definitions


/* == Optional author == */
// Add your name/username/identifier/alias etc. within "" ofc
$author = null;
// example:
#$author = "admin";


/* == Path of directory containing the articles == */
// so don't have to repeat this over and over, and another dir can be set easily
$articlePath = "content/articles/";


/* == Date format for the timestamp of blog entries == */
/*
 Pick one or create your own.
 Comment out the rest.
*/
#$dateFormat = "Y-m-d"; // eg. 2024-01-18
#$dateFormat = "Y-m-d h:i"; // eg. 2024-01-18 17:25
$dateFormat = "Y. F d."; // eg. 2024. january 18.
#$dateFormat = "Y. F d. h:i"; // eg. 2024. january 18. 17:25

/*
Month-day date format - used in the catalog when selecting a tag.
Feel free to define your own.
*/
$mdFormat = "m. d."; // eg. 01. 18.


/* == Gathering the blog entry files into an array == */
// this is done in three steps
// 1. scanning the directory for catalogs
// 2. filtering out the '.' and '..' directories from the catalogs, so we get only the files
// 3. reversing the order of the files, so when we loop through
// then the newer posts will be displayed first on the home page
$files = array_reverse(filterDots(scandir($articlePath)));


/* == Number of blog entries Per Page - for pagination == */
// how many blog entries are displayed on one page
$entryPerPage = 3;


/* == file number related variable calculations == */
// count the number of files gathered
$entryCount = count($files);
// calculate how many pages have to be generated
$numOfPages = ceil($entryCount / $entryPerPage); 
// rearrange the $files array into a multi level one
// each level contains one page worth of blog entries
$filesPages = array_chunk($files, $entryPerPage);



/* ============================== */
/* ==== Function Definitions ==== */
// when gathering the files in a directory into an array
// the method used also gathers the "." and ".." from that directory
// think the bash command ls -a, which also lists them
// "." = current directory
// ".." = one directory above
// this function below gets rid of those, since we can't use them for anything in this context
function filterDots($arr) {
    foreach ($arr as $value) {
        if ($value == '.' || $value == '..') {
            $position = array_search($value, $arr);
            array_splice($arr, $position, 1);
        }
    }
    return $arr;
}


// extracting the unix timestamp from a filename
// ie. the 1706967736 from 1706967736.html
function extractDate($filename) {
    $name = strstr($filename, '.', true); // the true is needed to extract the string before the "."
    return $name;
}


/* ===================================== */
/* ============== Entries ============== */

// creating one blog entry
// used in a loop on the home page, and used once on blog pages
function displayEntry($value, $path, $format, $auth) {
    // each entries are placed within an article html element
    // opening tag
    echo "<article>\n";
    // add a timestamp to the blog entry
    echo "<p class='tstamp'>" . date($format, extractDate($value)) ."</p>";
    // if author is specified then the name will be added here.
    if ($auth) {
        // just using a span
        echo "<span class='author'>by $auth</span>\n";
    }
    // put the contents of the blog entry into a variable
    $entry = file_get_contents($path . $value);
    // let's extract the date from the filename
    $timeStamp = extractDate($value);
    // Replacing the header with a link
    $openTag = "<h3><a href='blog.php?blog=$timeStamp'>"; // open html elems
    $closeTag = "</a></h3>"; // close html elems
    // now replace, have to use substr_replace in case user decides to add more h3 elements
    $entry = substr_replace($entry, $openTag, strpos($entry, "<h3>"), strlen("<h3>"));
    $entry = substr_replace($entry, $closeTag, strpos($entry, "</h3>"), strlen("</h3>"));

    // Checking for tags, note the e-tag html element was custom made for this purpose
    if (strpos($entry, "<e-tag>")) {
        $etagPos = strpos($entry, "<e-tag>");
        $tags = substr($entry, $etagPos); //get all the tags from the first to last into a string
        $entry = substr($entry, 0, $etagPos);
    }

    // Print the blog entry
    echo $entry;

    // if tags were found previously then make them into links
    if ($tags) {
        // envelope them into a separate div within the article
        echo "<div class='tags'>";
        // extract the name of the tags from the string containing html elements in 2 steps
        $tagsArray = getTags($tags);
        $tagNameArray = array_map("strip_tags", $tagsArray);
        // then loop through them and create a link from each
        // the link passes the tag name to the catalog via a GET request
        for ($i = 0; $i < count($tagsArray); $i++) {
            $tagsArray[$i] = substr_replace($tagsArray[$i], "<e-tag><a href='catalog.php?tag=$tagNameArray[$i]'>", strpos($tagsArray[$i], "<e-tag>"), strlen("<e-tag>"));
            $tagsArray[$i] = str_replace("</e-tag>", "</a></e-tag>", $tagsArray[$i]);
            echo $tagsArray[$i];
        }
        // after the loop is done and all the tags are "linkyfied", close the div
        echo "</div>";
    }
    // close the article
    echo "</article>\n";
}

// converting tags string into an array
function getTags($str) {
    // separate the string into an array, use newline char as delimiter
    // note: in theory on mac it's "\r" (carriage return), on windows "\r\n"
    $arr = explode("\n", $str);
    // for some reason it adds an empty element to the end of the array, that should be ignored
    $arr = array_slice($arr, 0, count($arr)-1);
    return $arr;
}


/* ===================================== */
/* ============== Catalog ============== */


// generate the entries/links for the catalog
// grouping them into years, creating a chronological order
function generateCatalog($arr, $path, $format) {
    $n = 1; // count each iteration of the following loop
    foreach ($arr as $value) {
        $timeStamp = extractDate($value); // create timestamp from the filename
        $year = date("Y", $timeStamp); // extract year from the timestamp
        $entry = file_get_contents($path . $value); // put the file contents into a variable
        $formattedDate = date($format, $timeStamp); // crate a formatted date from the timesampt
        $title = strip_tags(substr($entry, 0, strpos($entry, "\n"))); // extract the title from the file contents
        // create one entry from the date and the title, put the title into an "a" element
        // the link points to the specific blog, and passes its timestamp to the blog page
        $entryLink = "<span>$formattedDate - <a href='blog.php?blog=$timeStamp'>$title</a></span>";
        // if we are at the first iteration, eg. the first file
        if ($n == 1) {
            // then we note the year of this post, use it in testing
            $oldyear = $year;
            // and we open an article to list the blog entries which happened in the same year
            echo "<article><h3>$year</h3>\n";
        }
        // then if the year of the first file is larger than the year in the file we process
        // which can happen in the subsequent files (but obvioulsy not in the first)
        // and which indicates that we arrived to another year
        // which means we have to close the previous article of the previos year
        // and open a new article
        if ($oldyear > $year) {
            // update the year for tests
            $oldyear = $year;
            echo "</article>\n"; // close article of previous year
            echo "<article><h3>$year</h3>\n"; // open article for the new year
        }
        // display the link list
        echo $entryLink;
        // update the counter
        $n++;
    }
    // to complete the article of the very last iteration, we finish with closing that article
    echo "</article>\n";
}

// generate entries/links for the catalog
// filtered by tags
// this is simpler, only need to create one article
function generateCatalogByTag($arr, $path, $format, $t) {
    // open the article and add a header/title containing the tag name / search value essentially
    echo "<article><h3>Tag: $t</h3>\n";
    // change the tag name to inlcude the e-tag element, needed a bit later in the search
    // can't search the "naked" name in the file contents (in the blog text)
    // because it can be a legit part of the text body, resulting in false positive
    // ergo need to search with the html element
    $t = "<e-tag>$t</e-tag>";
    // loop through the files
    foreach ($arr as $value) {
        // get the text of the blog, contents of the file
        $entry = file_get_contents($path . $value);
        // check for the tag within the file (the content is a string)
        if (strpos($entry, $t)) {
            // if it's a hit, then do the following
            $timeStamp = extractDate($value); // create the timestamp
            $formattedDate = date($format, $timeStamp); // format the timestamp
            $title = strip_tags(substr($entry, 0, strpos($entry, "\n"))); // get the title
            // then assemble the whole thing together
            $entryLink =  "<span>$formattedDate - <a href='blog.php?blog=$timeStamp'>$title</a></span>";
            // print out the link
            echo $entryLink;
        }
    }
    // when the loop is done and all links are printed, close the article
    echo "</article>\n";
}


/* ======================================== */
/* ============== Pagination ============== */


// creating the page numbering to navigate through the blog entries on the home page
function generatePagination($last, $current) {
    // 1. create the container for the page numbers/links (they'll be <a> elements)
    echo "<div id='pagination'>";

    // 2. defining couple of variables for the loop
    // This is needed because we only display numbers with the offset of -3 and +3
    // from the current page we are on, so seven page numbers all in all
    // 
    // 2a. the $i where the counting starts
    if ($current-3 >= 1) {
        $i = $current-3;
    } else {
        $i = 1;
    }
    // 2b. the maximum number the loop runs, this is the max number of the pages
    if ($current+3 > $last) {
        $top = $last;
    } else {
        $top = $current + 3;
    }

    // 3. these two are needed so the first page is always displayed
    if ($current > 4) {
        echo "<a href='index.php?page=1'>1</a>";
    }
    if ($current > 5) {
        echo "<span>...</span>";
    }

    // 4. the loop which generates the main body of the numbers and prints them out
    for ($i ; $i <= $top ; $i++) {
        echo "<a href='index.php?page=$i'>$i</a>";
        
    }

    // 5. these two are needed so the last page is always displayed
    if ($current < $last-4) {
        echo "<span>...</span>";
    }
    if ($current < $last-3) {
        echo "<a href='index.php?page=$last'>$last</a>";
    }

    // 6. close the div
    echo "</div>";
}

?>
